#################################################
## some of the changes are from code
## Copyright 1999 Dr. Jens Oehlschlaegel-Akiyoshi
## Others are by BDR and MM
#################################################

pairs2 <-
function (x, x2, labels, panel = points, ...,
          lower.panel = panel, upper.panel = panel,
          diag.panel = NULL, text.panel = textPanel,
          label.pos = 0.5 + has.diag/3,
          cex.labels = NULL, font.labels = 1,
          row1attop = TRUE, gap = 1,
          cor = TRUE, cor.cex = 1.5, cor.col = "blue", cor.font = 2,
          cor.x = 0.1, cor.y = 0.1, cor.digits = 2,
          cor.offset = 0, cor.pos = 4)
{
    textPanel <-
        function(x = 0.5, y = 0.5, txt, cex, font)
            text(x, y, txt, cex = cex, font = font)

    localAxis <- function(side, x, y, xpd, bg, col=NULL, main, oma, ...) {
      ## Explicitly ignore any color argument passed in as
      ## it was most likely meant for the data points and
      ## not for the axis.
        if(side %%2 == 1) Axis(x, side=side, xpd=NA, ...)
        else Axis(y, side=side, xpd=NA, ...)
    }

    localPlot <- function(..., main, oma, font.main, cex.main) plot(...)
    localLowerPanel <- function(..., main, oma, font.main, cex.main)
        lower.panel(...)
    localUpperPanel <- function(..., main, oma, font.main, cex.main)
        upper.panel(...)

    dots <- list(...); nmdots <- names(dots)
    if (!is.matrix(x)) {
        x <- as.data.frame(x)
        for(i in seq(along=names(x))) {
            if(is.factor(x[[i]]) || is.logical(x[[i]]))
               x[[i]] <- as.numeric(x[[i]])
            if(!is.numeric(unclass(x[[i]])))
                stop("non-numeric argument to 'pairs2'")
        }
    } else if (!is.numeric(x)) stop("non-numeric argument to 'pairs2'")
    if (!is.matrix(x2)) {
        x2 <- as.data.frame(x2)
        for(i in seq(along=names(x2))) {
            if(is.factor(x2[[i]]) || is.logical(x2[[i]]))
               x2[[i]] <- as.numeric(x2[[i]])
            if(!is.numeric(unclass(x2[[i]])))
                stop("non-numeric argument to 'pairs2'")
        }
    } else if (!is.numeric(x2)) stop("non-numeric argument to 'pairs2'")
    panel <- match.fun(panel)
    if((has.lower <- !is.null(lower.panel)) && !missing(lower.panel))
        lower.panel <- match.fun(lower.panel)
    if((has.upper <- !is.null(upper.panel)) && !missing(upper.panel))
        upper.panel <- match.fun(upper.panel)
    if((has.diag  <- !is.null( diag.panel)) && !missing( diag.panel))
        diag.panel <- match.fun( diag.panel)

    if(row1attop) {
        tmp <- lower.panel; lower.panel <- upper.panel; upper.panel <- tmp
        tmp <- has.lower; has.lower <- has.upper; has.upper <- tmp
    }

    nc <- ncol(x)
    if (nc < 2) stop("only one column in the argument to 'pairs2'")
    nc2 <- ncol(x2)
    if (!(nc == nc2)) stop("x and x2 have different numbers of columns")
    has.labs <- TRUE
    if (missing(labels)) {
        labels <- colnames(x)
        if (is.null(labels)) labels <- paste("var", 1:nc)
    }
    else if(is.null(labels)) has.labs <- FALSE
    oma <- if("oma" %in% nmdots) dots$oma else NULL
    main <- if("main" %in% nmdots) dots$main else NULL
    if (is.null(oma)) {
        oma <- c(4, 4, 4, 4)
        if (!is.null(main)) oma[3] <- 6
    }
    opar <- par(mfrow = c(nc, nc), mar = rep.int(gap/2, 4), oma = oma)
    on.exit(par(opar))

    localPlot2 <- function (x)
        localPlot(x[, j], x[, i], xlab = "", ylab = "",
                  axes = FALSE, type = "n", ...)

    printCor <- function (x, i, j) {
        par(usr = c(0, 1, 0, 1))
        text(cor.x, cor.y,
             sprintf("%4.2f", cor(x[, j], x[, i])),
             cex = cor.cex, font = cor.font,
             col = cor.col, offset = cor.offset, pos = cor.pos)
    }

    for (i in if(row1attop) 1:nc else nc:1)
        for (j in 1:nc) {
            if (i < j)
                localPlot2(x2)
            else
                localPlot2(x)
            if(i == j || (i < j && has.lower) || (i > j && has.upper) ) {
                box()
                if(i == 1  && ((j > 1) || !has.upper || !has.lower ))
                    localAxis(1 + 2*row1attop, x2[, j], x2[, i], ...)
                if(i == nc && ((j < nc) || !has.upper || !has.lower ))
                    localAxis(3 - 2*row1attop, x[, j], x[, i], ...)
                if(j == 1  && ((i > 1) || !has.upper || !has.lower ))
                    localAxis(2, x[, j], x[, i], ...)
                if(j == nc && ((i < nc) || !has.upper || !has.lower ))
                    localAxis(4, x2[, j], x2[, i], ...)
                mfg <- par("mfg")
                if(i == j) {
                    if (has.diag) diag.panel(as.vector(x[, i]))
                    if (has.labs) {
                        par(usr = c(0, 1, 0, 1))
                        if(is.null(cex.labels)) {
                            l.wid <- strwidth(labels, "user")
                            cex.labels <- max(0.8, min(2, .9 / max(l.wid)))
                        }
                        text.panel(0.5, label.pos, labels[i],
                                   cex = cex.labels, font = font.labels)
                    }
                } else if(i < j) {
                    localLowerPanel(as.vector(x2[, j]), as.vector(x2[, i]), ...)
                    if (cor) printCor(x2, i, j)
                } else {
                    localUpperPanel(as.vector(x[, j]), as.vector(x[, i]), ...)
                    if (cor) printCor(x, i, j)
                }
                if (any(par("mfg") != mfg))
                    stop("the 'panel' function made a new plot")
            } else par(new = FALSE)

        }
    if (!is.null(main)) {
        font.main <- if("font.main" %in% nmdots) dots$font.main else par("font.main")
        cex.main <- if("cex.main" %in% nmdots) dots$cex.main else par("cex.main")
        mtext(main, 3, 3, TRUE, 0.5, cex = cex.main, font = font.main)
    }
    invisible(NULL)
}
